@@ -2,34 +2,40 @@
 import functools
 import inspect
 import itertools
-import pathlib
 from abc import ABCMeta
 from abc import abstractmethod
 from pathlib import Path
 from typing import Any
+from typing import Callable
 from typing import Dict
+from typing import Generator
 from typing import Iterable
 from typing import List
+from typing import Optional
+from typing import Set
 from typing import Tuple
+from typing import TYPE_CHECKING
 from typing import Union
 
 import attr
-from _pytask.console import escape_squared_brackets
 from _pytask.exceptions import NodeNotCollectedError
 from _pytask.exceptions import NodeNotFoundError
-from _pytask.mark import get_marks_from_obj
-from _pytask.path import find_closest_ancestor
-from _pytask.path import find_common_ancestor
-from _pytask.path import relative_to
-from _pytask.shared import find_duplicates
+from _pytask.mark_utils import get_marks_from_obj
+from _pytask.session import Session
 
 
-def depends_on(objects: Union[Any, Iterable[Any]]) -> Union[Any, Iterable[Any]]:
+if TYPE_CHECKING:
+    from _pytask.mark import Mark
+
+
+def depends_on(
+    objects: Union[Any, Iterable[Any], Dict[Any, Any]]
+) -> Union[Any, Iterable[Any], Dict[Any, Any]]:
     """Specify dependencies for a task.
 
     Parameters
     ----------
-    objects : Union[Any, Iterable[Any]]
+    objects : Union[Any, Iterable[Any], Dict[Any, Any]]
         Can be any valid Python object or an iterable of any Python objects. To be
         valid, it must be parsed by some hook implementation for the
         :func:`pytask.hookspecs.pytask_collect_node` entry-point.
@@ -38,12 +44,14 @@ def depends_on(objects: Union[Any, Iterable[Any]]) -> Union[Any, Iterable[Any]]:
     return objects
 
 
-def produces(objects: Union[Any, Iterable[Any]]) -> Union[Any, Iterable[Any]]:
+def produces(
+    objects: Union[Any, Iterable[Any], Dict[Any, Any]]
+) -> Union[Any, Iterable[Any], Dict[Any, Any]]:
     """Specify products of a task.
 
     Parameters
     ----------
-    objects : Union[Any, Iterable[Any]]
+    objects : Union[Any, Iterable[Any], Dict[Any, Any]]
         Can be any valid Python object or an iterable of any Python objects. To be
         valid, it must be parsed by some hook implementation for the
         :func:`pytask.hookspecs.pytask_collect_node` entry-point.
@@ -55,19 +63,37 @@ def produces(objects: Union[Any, Iterable[Any]]) -> Union[Any, Iterable[Any]]:
 class MetaNode(metaclass=ABCMeta):
     """Meta class for nodes."""
 
+    name: str
+    path: Path
+
     @abstractmethod
-    def state(self):
-        """Return a value which indicates whether a node has changed or not."""
-        pass
+    def state(self) -> Optional[str]:
+        ...
 
 
 class MetaTask(MetaNode):
     """The base class for tasks."""
 
+    base_name: str
+    name: str
+    short_name: Optional[str]
+    markers: List["Mark"]
+    depends_on: Dict[str, MetaNode]
+    produces: Dict[str, MetaNode]
+    path: Path
+    function: Optional[Callable[..., Any]]
+    attributes: Dict[Any, Any]
+    _report_sections: List[Tuple[str, str, str]]
+
     @abstractmethod
-    def execute(self):
-        """Execute the task."""
-        pass
+    def execute(self) -> None:
+        ...
+
+    @abstractmethod
+    def add_report_section(
+        self, when: str, key: str, content: str  # noqa: U100
+    ) -> None:
+        ...
 
 
 @attr.s
@@ -80,34 +106,42 @@ class PythonFunctionTask(MetaTask):
     """str: The unique identifier for a task."""
     path = attr.ib(type=Path)
     """pathlib.Path: Path to the file where the task was defined."""
-    function = attr.ib(type=callable)
-    """callable: The task function."""
-    depends_on = attr.ib(factory=dict)
-    """Optional[List[MetaNode]]: A list of dependencies of task."""
-    produces = attr.ib(factory=dict)
-    """List[MetaNode]: A list of products of task."""
-    markers = attr.ib(factory=list)
+    function = attr.ib(type=Callable[..., Any])
+    """Callable[..., Any]: The task function."""
+    short_name = attr.ib(default=None, type=Optional[str], init=False)
+    """str: The shortest uniquely identifiable name for task for display."""
+    depends_on = attr.ib(factory=dict, type=Dict[str, MetaNode])
+    """Dict[str, MetaNode]: A list of dependencies of task."""
+    produces = attr.ib(factory=dict, type=Dict[str, MetaNode])
+    """Dict[str, MetaNode]: A list of products of task."""
+    markers = attr.ib(factory=list, type="List[Mark]")
     """Optional[List[Mark]]: A list of markers attached to the task function."""
-    keep_dict = attr.ib(default=False)
+    keep_dict = attr.ib(factory=dict, type=Dict[str, bool])
     """Dict[str, bool]: Should dictionaries for single nodes be preserved?"""
-    _report_sections = attr.ib(factory=list)
-    """List[Tuple[str]]: A list of reports with entries for when, what, and content."""
-    attributes = attr.ib(factory=dict)
+    _report_sections = attr.ib(factory=list, type=List[Tuple[str, str, str]])
+    """List[Tuple[str, str, str]]: Reports with entries for when, what, and content."""
+    attributes = attr.ib(factory=dict, type=Dict[Any, Any])
     """Dict[Any, Any]: A dictionary to store additional information of the task."""
 
+    def __attrs_post_init__(self: "PythonFunctionTask") -> None:
+        if self.short_name is None:
+            self.short_name = self.name
+
     @classmethod
-    def from_path_name_function_session(cls, path, name, function, session):
+    def from_path_name_function_session(
+        cls, path: Path, name: str, function: Callable[..., Any], session: Session
+    ) -> "PythonFunctionTask":
         """Create a task from a path, name, function, and session."""
         keep_dictionary = {}
 
         objects = _extract_nodes_from_function_markers(function, depends_on)
-        nodes, keep_dict = _convert_objects_to_node_dictionary(objects, "depends_on")
-        keep_dictionary["depends_on"] = keep_dict
+        nodes, keep_dict_de = _convert_objects_to_node_dictionary(objects, "depends_on")
+        keep_dictionary["depends_on"] = keep_dict_de
         dependencies = _collect_nodes(session, path, name, nodes)
 
         objects = _extract_nodes_from_function_markers(function, produces)
-        nodes, keep_dict = _convert_objects_to_node_dictionary(objects, "produces")
-        keep_dictionary["produces"] = keep_dict
+        nodes, keep_dict_prod = _convert_objects_to_node_dictionary(objects, "produces")
+        keep_dictionary["produces"] = keep_dict_prod
         products = _collect_nodes(session, path, name, nodes)
 
         markers = [
@@ -127,16 +161,16 @@ class PythonFunctionTask(MetaTask):
             keep_dict=keep_dictionary,
         )
 
-    def execute(self):
+    def execute(self) -> None:
         """Execute the task."""
         kwargs = self._get_kwargs_from_task_for_function()
         self.function(**kwargs)
 
-    def state(self):
+    def state(self) -> str:
         """Return the last modified date of the file where the task is defined."""
         return str(self.path.stat().st_mtime)
 
-    def _get_kwargs_from_task_for_function(self):
+    def _get_kwargs_from_task_for_function(self) -> Dict[str, Any]:
         """Process dependencies and products to pass them as kwargs to the function."""
         func_arg_names = set(inspect.signature(self.function).parameters)
         kwargs = {}
@@ -153,7 +187,7 @@ class PythonFunctionTask(MetaTask):
 
         return kwargs
 
-    def add_report_section(self, when: str, key: str, content: str):
+    def add_report_section(self, when: str, key: str, content: str) -> None:
         if content:
             self._report_sections.append((when, key, content))
 
@@ -162,18 +196,18 @@ class PythonFunctionTask(MetaTask):
 class FilePathNode(MetaNode):
     """The class for a node which is a path."""
 
-    name = attr.ib()
+    name = attr.ib(type=str)
     """str: Name of the node which makes it identifiable in the DAG."""
 
-    value = attr.ib()
+    value = attr.ib(type=Path)
     """Any: Value passed to the decorator which can be requested inside the function."""
 
-    path = attr.ib()
+    path = attr.ib(type=Path)
     """pathlib.Path: Path to the FilePathNode."""
 
     @classmethod
     @functools.lru_cache()
-    def from_path(cls, path: pathlib.Path):
+    def from_path(cls, path: Path) -> "FilePathNode":
         """Instantiate class from path to file.
 
         The `lru_cache` decorator ensures that the same object is not collected twice.
@@ -183,7 +217,7 @@ class FilePathNode(MetaNode):
             raise ValueError("FilePathNode must be instantiated from absolute path.")
         return cls(path.as_posix(), path, path)
 
-    def state(self):
+    def state(self) -> Optional[str]:
         """Return the last modified date for file path."""
         if not self.path.exists():
             raise NodeNotFoundError
@@ -192,8 +226,8 @@ class FilePathNode(MetaNode):
 
 
 def _collect_nodes(
-    session, path: Path, name: str, nodes: Dict[str, Union[str, Path]]
-) -> Dict[str, Path]:
+    session: Session, path: Path, name: str, nodes: Dict[str, Union[str, Path]]
+) -> Dict[str, MetaNode]:
     """Collect nodes for a task.
 
     Parameters
@@ -209,7 +243,7 @@ def _collect_nodes(
 
     Returns
     -------
-    Dict[str, Path]
+    Dict[str, MetaNode]
         A dictionary of node names and their paths.
 
     Raises
@@ -226,8 +260,8 @@ def _collect_nodes(
         )
         if collected_node is None:
             raise NodeNotCollectedError(
-                f"'{node}' cannot be parsed as a dependency or product for task "
-                f"'{name}' in '{path}'."
+                f"{node!r} cannot be parsed as a dependency or product for task "
+                f"{name!r} in {path!r}."
             )
         else:
             collected_nodes[node_name] = collected_node
@@ -235,7 +269,9 @@ def _collect_nodes(
     return collected_nodes
 
 
-def _extract_nodes_from_function_markers(function, parser):
+def _extract_nodes_from_function_markers(
+    function: Callable[..., Any], parser: Callable[..., Any]
+) -> Generator[Any, None, None]:
     """Extract nodes from a marker.
 
     The parser is a functions which is used to document the marker with the correct
@@ -249,7 +285,9 @@ def _extract_nodes_from_function_markers(function, parser):
         yield parsed
 
 
-def _convert_objects_to_node_dictionary(objects, when):
+def _convert_objects_to_node_dictionary(
+    objects: Any, when: str
+) -> Tuple[Dict[Any, Any], bool]:
     """Convert objects to node dictionary."""
     list_of_tuples, keep_dict = _convert_objects_to_list_of_tuples(objects, when)
     _check_that_names_are_not_used_multiple_times(list_of_tuples, when)
@@ -257,7 +295,9 @@ def _convert_objects_to_node_dictionary(objects, when):
     return nodes, keep_dict
 
 
-def _convert_objects_to_list_of_tuples(objects, when: str):
+def _convert_objects_to_list_of_tuples(
+    objects: Union[Any, Tuple[Any, Any], List[Any], List[Tuple[Any, Any]]], when: str
+) -> Tuple[List[Tuple[Any, ...]], bool]:
     """Convert objects to list of tuples.
 
     Examples
@@ -298,7 +338,9 @@ def _convert_objects_to_list_of_tuples(objects, when: str):
     return out, keep_dict
 
 
-def _check_that_names_are_not_used_multiple_times(list_of_tuples, when):
+def _check_that_names_are_not_used_multiple_times(
+    list_of_tuples: List[Tuple[Any, ...]], when: str
+) -> None:
     """Check that names of nodes are not assigned multiple times.
 
     Tuples in the list have either one or two elements. The first element in the two
@@ -326,8 +368,8 @@ def _check_that_names_are_not_used_multiple_times(list_of_tuples, when):
 
 
 def _convert_nodes_to_dictionary(
-    list_of_tuples: List[Tuple[str]],
-) -> Dict[str, Union[str, Path]]:
+    list_of_tuples: List[Tuple[Any, ...]]
+) -> Dict[Any, Any]:
     """Convert nodes to dictionaries.
 
     Examples
@@ -358,7 +400,7 @@ def _convert_nodes_to_dictionary(
     return nodes
 
 
-def create_task_name(path: Path, base_name: str):
+def create_task_name(path: Path, base_name: str) -> str:
     """Create the name of a task from a path and the task's base name.
 
     Examples
@@ -371,38 +413,23 @@ def create_task_name(path: Path, base_name: str):
     return path.as_posix() + "::" + base_name
 
 
-def reduce_node_name(node, paths: List[Path]):
-    """Reduce the node name.
+def find_duplicates(x: Iterable[Any]) -> Set[Any]:
+    """Find duplicated entries in iterable.
 
-    The whole name of the node - which includes the drive letter - can be very long
-    when using nested folder structures in bigger projects.
-
-    Thus, the part of the name which contains the path is replaced by the relative
-    path from one path in ``session.config["paths"]`` to the node.
+    Examples
+    --------
+    >>> find_duplicates(["a", "b", "a"])
+    {'a'}
+    >>> find_duplicates(["a", "b"])
+    set()
 
     """
-    ancestor = find_closest_ancestor(node.path, paths)
-    if ancestor is None:
-        try:
-            ancestor = find_common_ancestor(node.path, *paths)
-        except ValueError:
-            ancestor = node.path.parents[-1]
+    seen = set()
+    duplicates = set()
 
-    if isinstance(node, MetaTask):
-        shortened_path = relative_to(node.path, ancestor)
-        raw_name = create_task_name(shortened_path, node.base_name)
-        name = escape_squared_brackets(raw_name)
-    elif isinstance(node, MetaNode):
-        name = relative_to(node.path, ancestor).as_posix()
-    else:
-        raise ValueError(f"Unknown node {node} with type '{type(node)}'.")
+    for i in x:
+        if i in seen:
+            duplicates.add(i)
+        seen.add(i)
 
-    return name
-
-
-def reduce_names_of_multiple_nodes(names, dag, paths):
-    """Reduce the names of multiple nodes in the DAG."""
-    return [
-        reduce_node_name(dag.nodes[n].get("node") or dag.nodes[n].get("task"), paths)
-        for n in names
-    ]
+    return duplicates
